<?php 
/**
 * Plugin Name: AutoDescription
 * Plugin URI: https://hostmijnpagina.nl/
 * Description: Automatically adds a description if previously empty based upon content.
 * Version: 1.0.1
 * Author: Sybre Waaijer
 * Author URI: https://cyberwire.nl/
 * Text Domain: AutoDescription
 */

/** 
 * Fully supports Genesis themes, this plugin is build upon it.
 * Doesn't have full support for other SEO plugins/themes but should work.
 * Please notify me if you notice an issue with a specific theme or plugin.
 */

function ad_locale_init() {
	$plugin_dir = basename(dirname(__FILE__));
	load_plugin_textdomain( 'AutoDescription', false, $plugin_dir . '/language/');
}
add_action('plugins_loaded', 'ad_locale_init');
 
//* Get the excerpt
function hmpl_get_excerpt_by_id($get_hmpl_excerpt) {	
    $get_hmpl_excerpt = '';
	
	if (!is_404() && !is_search()) {
		global $post_id;
		$the_post = get_post($post_id);
		$get_hmpl_excerpt = $the_post->post_content;
	}
	
    $get_hmpl_excerpt = esc_attr(strip_tags(strip_shortcodes($get_hmpl_excerpt)));
	
	$get_hmpl_excerpt = str_replace(array("\r\n", "\r", "\n"), "\n", $get_hmpl_excerpt);
	$lines = explode("\n", $get_hmpl_excerpt);
	$new_lines = array();
	
	foreach ($lines as $i => $line) {
		if(!empty($line))
			$new_lines[] = trim($line) . ' ';
	}
	
	$get_hmpl_excerpt = implode($new_lines);
	
    return $get_hmpl_excerpt;
}
 
//* Create description
function hmpl_og_description($hmpl_og_description) {
	global $wp_query;

	$hmpl_og_description = '';
	
	//* Genesis only, checks if description is present
	$theme_info = wp_get_theme()->get('Template');

	if( $theme_info == 'genesis' ) {
		if ( is_front_page() ) {
			$hmpl_og_description = genesis_get_seo_option( 'home_description' ) ? genesis_get_seo_option( 'home_description' ) : get_bloginfo( 'description' );
		}
		if ( is_singular() ) {
			if ( genesis_get_custom_field( '_genesis_description' ) )
				$hmpl_og_description = genesis_get_custom_field( '_genesis_description' );
		}
		if ( is_category() ) {
			$term = $wp_query->get_queried_object();
			$hmpl_og_description = ! empty( $term->meta['description'] ) ? $term->meta['description'] : '';
		}
		if ( is_tag() ) {
			$term = $wp_query->get_queried_object();
			$hmpl_og_description = ! empty( $term->meta['description'] ) ? $term->meta['description'] : '';
		}
		if ( is_tax() ) {
			$term = get_term_by( 'slug', get_query_var( 'term' ), get_query_var( 'taxonomy' ) );
			$hmpl_og_description = ! empty( $term->meta['description'] ) ? wp_kses_stripslashes( wp_kses_decode_entities( $term->meta['description'] ) ) : '';
		}
		if ( is_author() ) {
			$user_description = get_the_author_meta( 'meta_description', (int) get_query_var( 'author' ) );
			$hmpl_og_description = $user_description ? $user_description : '';
		}
		if ( is_post_type_archive() && genesis_has_post_type_archive_support() ) {
			$hmpl_og_description = genesis_get_cpt_option( 'description' ) ? genesis_get_cpt_option( 'description' ) : '';
		}
	}

	if ( empty ($hmpl_og_description) ) { //* WARNING: HEAVY CODE (adds .5s processing time with 241k characters on 6-core 2.4GHz prefork)
		global $post,$get_hmpl_excerpt;
		$hmpl_title = get_the_title($post);
		$hmpl_blogname = get_bloginfo('name');
		$hmpl_excut = '';
		$hmpl_on = __('on', 'AutoDescription'); // Post Title "on" Blog Name
		
		$hmpl_excerpt = hmpl_get_excerpt_by_id($get_hmpl_excerpt);
		$hmpl_maxcharlength = 160 - mb_strlen($hmpl_title . $hmpl_blogname);

		if ( mb_strlen( $hmpl_excerpt ) > $hmpl_maxcharlength ) {
			$subex = mb_substr( $hmpl_excerpt, 0, $hmpl_maxcharlength - 3 );
			$exwords = explode( ' ', $subex );
			$excut = - ( mb_strlen( $exwords[ count( $exwords ) - 1 ] ) );
			if ( $excut < 0 ) {
				$hmpl_excerpt = mb_substr( $subex, 0, $excut );
			} else {
				$hmpl_excerpt = $subex;
			}
			$hmpl_excut = '...';
		}
		$hmpl_excerpt = str_replace(' ...', '...', $hmpl_excerpt . $hmpl_excut);
		
		$hmpl_og_description = sprintf( '%s %s %s - %s', $hmpl_title, $hmpl_on, $hmpl_blogname, $hmpl_excerpt);		
	}	
	
	//* Make sure no spaces are added at the end, adds " symbol
	$hmpl_og_description = $hmpl_og_description . '"';
	$hmpl_og_description = str_replace(' "', '"', $hmpl_og_description);
	
	return $hmpl_og_description;
}

function hmpl_og_locale($hmpl_og_locale) {
	$hmpl_og_locale = get_locale();	
	return $hmpl_og_locale;
}

//* Overwrite the old description with an automated one
function hmpl_blog_description($hmpl_blog_description) {
	global $hmpl_og_description;
	
	$hmpl_blog_description = hmpl_og_description($hmpl_og_description);
	
	if ( $hmpl_blog_description )
		echo '<meta name="description" content="' . $hmpl_blog_description . ' />' . "\n"; //Removed the " deliberately.
}

//* Get the title for og:title
function hmpl_og_title($hmpl_og_title) {
	global $post;
	
	$hmpl_blogname = get_bloginfo('name');
	$hmpl_title = get_the_title($post);
	$hmpl_tagline = get_bloginfo( 'description', 'raw');
	
	$hmpl_og_title = $hmpl_title . ' - ' . $hmpl_blogname;
	
	if ( is_front_page() )
		$hmpl_og_title = sprintf( '%s - %s', $hmpl_blogname, $hmpl_tagline);	
		
	return $hmpl_og_title;
}

//* Output custom header og meta NOTE: removed the " in og:description deliberately.
function add_hmpl_meta_tags() {
	global $wp,$hmpl_og_title,$hmpl_og_description,$hmpl_og_locale;
?>	<meta property="og:image" content="<?php echo wp_make_link_relative(get_header_image()); ?>" />
	<meta property="og:locale" content="<?php echo hmpl_og_locale($hmpl_og_locale); ?>" />
	<meta property="og:type" content="website" />
	<meta property="og:title" content="<?php echo esc_attr(hmpl_og_title($hmpl_og_title)); ?>" />
	<meta property="og:description" content="<?php echo hmpl_og_description($hmpl_og_description); // Misses " deliberately ?> />
	<meta property="og:url" content="<?php echo home_url(add_query_arg(array(),$wp->request)); ?>" />
	<meta property="og:site_name" content="<?php echo get_bloginfo('name'); ?>" /> <?php
}

//* Output in header
add_action( 'init', 'hmpl_auto_description_output', 10 );

//* # Filter 'autodescription_mu_load'
//* To disable, add this in a plugin or your theme's functions.php: add_filter('autodescription_mu_load', 'disable'); 
function hmpl_auto_description_output() {
	
	$ad_mu_load = apply_filters( 'autodescription_mu_load', 1 );
	
	if ($ad_mu_load == 1) {
		if (!is_user_logged_in()) {
			//* Genesis only, checks if description is present
			$theme_info = wp_get_theme()->get('Template');

			if( $theme_info == 'genesis' ) {
				remove_action( 'genesis_meta', 'genesis_seo_meta_description', 10 );
				add_action( 'genesis_meta', 'hmpl_blog_description' );
				add_action( 'genesis_meta', 'add_hmpl_meta_tags' );
			} else {
				add_action( 'wp_head', 'hmpl_blog_description' );
				add_action( 'wp_head', 'add_hmpl_meta_tags' );
			}
		}
	}
}